
'*******************************************************************
' Stack Fits Images.txt
' James L. Jones, 2005                                                               
'                                                                                          
'This script utilizes MaxIm to stack images in the Fits Format.          
'The images to be stacked are placed in a folder.  Nothing else        
'Can be in the folder.  Drag and drop the image folder to the           
'script icon.  The script will tell you how many images you have      
'in the folder and ask how many images you want in each stack.                                                                                                 *
'Sit back and watch.                                                                                                             
'The stacked images and a log file will be placed in the image folder.
' 
'Thanks to Bob Denny for showing me the Drag and Drop thing.
'                                                                                        
'******************************************************************

Option Explicit
Dim Doc, Apps, AllDocs, Path, collArgs, fso, f, fc, count, fl, SavePath
Dim Images(1500), Img(1200), ImgDup(1200), i, ii, j, Item(10), N, Shift, jint, jmod, k, StackIndex, ExpTime, ISOSPEED
Dim LineTxt(1500), ImageText(1500), DateTime, MidPoint, MidPointCor, MidPointCorTemp, ts, cso, TxtLog, indx, jj, kk
Dim Name, TimeShift, DateTimeShiftTemp, DateTimeShift

Set collArgs = WScript.Arguments
Call Drag_Drop()

'*********************************************
'     Get Collection of file Names                        
'*********************************************
Set fso = CreateObject("Scripting.FileSystemObject")
Set f = fso.GetFolder(collArgs(0))
Set fc = f.Files
Call MoveColl()
Call InputData()
Call Calculate()

'**********************************************************
'    jint = number of whole stacks                       
'    N = Number of images in each whole stack            
'    jmod = number of images in partial stack, if any    
'    k = first image in stack                            
'    k+N-1 = Last image in stack                         
'    ii = MaxIm Image Number                             
'    i = Image being oppened from Images()               
'**********************************************************
Indx = 0
k = 0
Call CloseAllImages()
For StackIndex = 0 To jint-1
	ii = 0
	
		For i = k To k+N-1
			Call  OpenImage(i, ii)
			ii = ii + 1
		Next
	k = k + Shift
	Call Stack_Save()	
	Call CloseAllImages()
Next
Call CreateLog()
ts.close
Msgbox "Finished!", , "Finished"


'*********************************************
'                Drag and Drop              
'*********************************************
Sub Drag_Drop()
count = 0
If collArgs.Count = 0 Then
      WScript.Echo _
"Drag a folder of FITS images to this icon to stack."
    	WScript.Quit
End If
End Sub

'*****************************************************************
'    Move collection of file names (fc) to array (Images())     
'*****************************************************************
Sub MoveColl()
count = fc.Count 
i = 0
For Each fl In fc
	Images(i) = fl.name
	i=i+1
Next
End Sub

'***********************************************
'    Input number of images in each stack      
'***********************************************
Sub InputData()
N = InputBox("There are " & fc.count & " images in the folder." & vbCrlf & "How many do you want In Each stack?","Enter Value", fc.count)
Shift = InputBox("What shift do you want to Next stack?","Enter Value", N)
Name =  InputBox("Name for stacks?","Enter Value", "stack")
TimeShift = cdate(InputBox("Put difference in hours between Time in Exif and UT (your Timezone and daylightsaving):","Enter Value", 0)/24)
End Sub

'******************************************
'          Subroutine CloseAllImages               
'******************************************
Sub CloseAllImages()
Set Apps = CreateObject("MaxIm.Application")
Apps.CloseAll()
End Sub

'***********************************************
'           Subroutine Calculate              
' Calculate number of stacks (jint) and the    
' number of images In the partial stack       
'***********************************************
Sub Calculate()
	jint = (count - N) \ Shift + 1	'jint = number of whole stacks
	jmod = count Mod N		'jmod = number of images in partial stack, if any
End Sub

'********************************
'    Subroutine Stack_Save     
'********************************
Sub Stack_save()
SavePath = collArgs(0) & "\" & Name & StackIndex & ".fts"
Set Img(ii) = CreateObject("MaxIm.Document")

	'Document.CombineImages ( Mode, Bicubic, Output, Normalize[, Images ]) :
		'AlignType Mode: 0 - autocorrelation, 1 - automatic matching of star brightnesses and separations, 2 - perform no alignment, 3 - WCS information in the FITS header, 5 - aligns centroids of the brightest star found in each image
		'CombineType Output: 0 - sum, 1 - average, 2 - median, 3 - sigma-clip algorithm, 4 - standard-deviation masking
		'Boolean Normalize - prenormalizes images so that medians are comparable (ignored if output is calculated by sum or average methods)
		'Collection Images - [optional] a Collection or Dictionary object whose members are the images to be aligned
Img(ii).CombineImages 1, False, 1, False

Call ReadKeyWords2()  
ImageText(Indx) = Name & StackIndex & ".fts" & "    DATE-OBS: " & DateTime & "    MIDPOINT: " & MidPoint & "     ExpTime in sec: " & ExpTime & "     " & ISOSPEED  

	'Document.SaveFile ( FilePath, FileFormat, AutoStretch[, SizeFormat, CompressionType ] :
		'FileFormat: 3 -  FITS, 4 - simple Raw data;
		'SizeFormat: 0 - 8 bits,  1 - 16 bits, 2 - 32 bits, 3 - 4bytes in IEEE floating point format
Img(ii).SaveFile SavePath, 3, False, 3, False  

Indx = Indx + 1		
End Sub

'********************************
'     Subroutine OpenImage()   
'********************************
Sub OpenImage(i, ii)
Set Img(ii) = CreateObject("MaxIm.Document")
Path = collArgs(0) & "\" & Images(i)
Img(ii).OpenFile (Path)
'Img(ii).Calibrate 'calibration
Set ImgDup(ii) = Img(ii).Duplicate
Img(ii).ExtractBayerPlane 2
ImgDup(ii).ExtractBayerPlane 3
Call ReadKeyWords()
	'WScript.Echo i, ii, StackIndex, Images(i)
ImageText(Indx) = Images(i) &  "     " & DateTime 
	'WScript.Echo Indx, ImageText(Indx)
Indx = Indx + 1	
Set ImgDup(ii) = Nothing
End Sub

'*********************************
'   Subroutine ReadKeyWords()   
'*********************************
Sub ReadKeyWords()
DateTime = (Img(ii).GetFITSKey("DATE-OBS"))  
MidPoint = (Img(ii).GetFITSKey("MIDPOINT"))  
ExpTime = (Img(ii).GetFITSKey("EXPTIME"))  
End Sub

'*********************************
'   Subroutine ReadKeyWords2()   
'*********************************
Sub ReadKeyWords2()  
Call ReadKeyWords()  
ISOSPEED = (Img(ii).GetFITSKey("ISOSPEED"))  
Call MidPointCorrector(MidPoint)  
Img(ii).SetFITSKey "MIDPOINT",MidPointCor  
MidPoint = (Img(ii).GetFITSKey("MIDPOINT"))
Call DateTimeShifter(DateTime)  
Img(ii).SetFITSKey "DATE-OBS",DateTimeShift
DateTime = (Img(ii).GetFITSKey("DATE-OBS"))  
End Sub

'*********************************
'   Subroutine MidPointCorrector()   
'*********************************
Sub MidPointCorrector(MidPoint)
MidPointCorTemp = cdate(Replace(MidPoint,"T"," ")) + cdate(ExpTime / 2 / 86400) - TimeShift  
MidPointCor = Year(MidPointCorTemp) & "-" & Month(MidPointCorTemp) & "-" & Day(MidPointCorTemp) & "T" & DatePart("h",MidPointCorTemp) & ":" & DatePart("n",MidPointCorTemp) & ":" & DatePart("s",MidPointCorTemp)
End Sub

'*********************************
'   Subroutine MidPointCorrector()   
'*********************************
Sub DateTimeShifter(DateTime)
DateTimeShiftTemp = cdate(Replace(DateTime,"T"," ")) - TimeShift  
DateTimeShift = Year(DateTimeShiftTemp) & "-" & Month(DateTimeShiftTemp) & "-" & Day(DateTimeShiftTemp) & "T" & DatePart("h",DateTimeShiftTemp) & ":" & DatePart("n",DateTimeShiftTemp) & ":" & DatePart("s",DateTimeShiftTemp)
End Sub

'*********************************
'   Subroutine CreateLog()       
'*********************************
Sub CreateLog()
Set cso = CreateObject("Scripting.FileSystemObject")
TxtLog = collArgs(0) & "\" & Name & "_log.txt"
Set ts = cso.OpenTextFile(TxtLog, 2, True)
For kk = 0 To Indx
	ts.WriteLine(ImageText(kk))
			
Next		
End Sub